<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="utf-8" />
        <title>
            {% block title %}{% endblock %}
        </title>
    </head>
    <body>
        {% block content %}
        {% endblock content %}
    </body>
</html>